#!/bin/sh
#
# Name: opus_catalog.s
#
# Description: Generates an OPUS catalog (.cat) file for a reference
#              file delivery.  If a deliveryNumber is given as an argument,
#              the .cat file is for that delivery, otherwise the latest
#              delivery number is read from the database and the .cat file
#              is for that delivery.
#
# Called by:   opus_catalog
#
# Requires: env var CDBS_SERVER
#           env var CDBS_DB
#
# Usage:    opus_catalog.s [deliveryNumber] 
#                           uses latest deliveryNumber if not supplied
#
# Returns: 0 - no problems
#          1 - error in generating the OPUS .cat file
#
#------------------------------------------------------------------------
export CDBS_DB CDBS_SERVER
echo "starting opus_catalog"
echo "database " ${CDBS_DB?"CDBS_ERROR: CDBS_DB is undefined"}
echo "server " ${CDBS_SERVER?"CDBS_ERROR: CDBS_SERVER is undefined"}
date
#
# obtain the deliveryNumber, either from the command line or from the database 
if test $# -lt 1
then
  # no deliveryNumber argument, so determine the latest opus delivery number
  delivery_number=`get_current_delivery_number.s | awk '/^ *[0-9]/{print $1}'`
else
  # use deliveryNumber from command line arg
  delivery_number=$1
fi
echo "delivery_number=" $delivery_number
#
# Determine whether the ga date has been set in the delivery
w=`opus_catalog_ga_date1.s $delivery_number | fgrep "ERROR" | wc -c`
if test $w -gt 0
then
   # there was an ERROR in determing the ga date
   opus_catalog_ga_date1.s $delivery_number
   echo "No catalog was generated."
   exit 1
fi
#
# Determine the instrument abbreviation for this delivery
instr=`opus_catalog_instrument_abbreviation.s $delivery_number | awk '/^ *[a-z] *$/{print $1}'`
echo "instr=" $instr
#
# form the name of the catalog file
cat_file="opus_"$delivery_number"_"$instr".cat"
echo "catalog file=" $cat_file
#
# populate the catalog file for this delivery
opus_catalog_inserted_deleted.s $delivery_number | awk '/^ *OPUS_ROW/{print $2, $3, $4}' > $cat_file
#
# ensure catalog file is not empty
if test -s $cat_file
then
  exit 0
else
  echo "ERROR: $cat_file is EMPTY."
  exit 1
fi
