#----------------------------------------------------------------------------
#
# module:  opusutil
#
# General purpose OPUS-Python functions
#
#         NOTE: Needed so CDBS AUTO tests can run properly.
#               We would rather NOT put this code in the AUTO tree
#               itself, and since CDBS doesn't use SHARE, it gets
#               its own copy to support AUTO tests.
#
# 01/20/05  52348  Sherbert Remove "MSG_DEGUG" to obviate its use
# 12/15/05  54226  Sontag   Added UsingLvl() and __all_valid_levels
# 06/22/07  57369  Sontag   Remove use of deprecated string module
# 11/02/10  67113  MSwam    add DateTrim() for MSSQL DB date read/writeback
#---------------------------------------------------------------------------
import os
import sys
import time
import re
import osfile_stretch

SUBSECONDS_REGEX = "(.+)(\.)([0-9]+)($)" # (anychars, dot, any digits, EOS)
SUBSECONDS_MATCH = re.compile(SUBSECONDS_REGEX)

# Read the environment setting for the PrintMsg report level
# or default if not defined
#
# This section is done only once, during the "import opusutil" statement
#
try:
  _report_level_val = os.environ['MSG_REPORT_LEVEL']
except KeyError:
  _report_level_val = "MSG_INFO"
#
# define the active levels for message reporting
if _report_level_val == "MSG_NONE":
  _active_levels = []
elif _report_level_val == "MSG_FATAL":
  _active_levels = ["F"]
elif _report_level_val == "MSG_ERROR":
  _active_levels = ["F","E"]
elif _report_level_val[0:8] == "MSG_WARN":
  _active_levels = ["F","E","W"]
elif (_report_level_val == "MSG_DIAG"  or
      _report_level_val == "MSG_ALL" ):
  _active_levels = ["F","E","W","I","D"]
else: # default
  _active_levels = ["F","E","W","I"]
#
# define the list of all possible levels
__all_valid_levels = ["F","E","W","I","D"]
#
# set a default message trailer device
_trl_fd = None

#--------------------------------------------------------------------------
#
# name:    OpenTrl
#
# purpose: Assigns a file object for trailer messages.
#
# input:   filespec - name of trailer file
#
# return:  none
#
# 01/13/03  46981  MSwam    first version
#---------------------------------------------------------------------------
def OpenTrl (filespec):
  global _trl_fd
  _trl_fd = open(filespec, 'a')

#--------------------------------------------------------------------------
#
# name:    CloseTrl
#
# purpose: Closes an open trailer file.
#
# input:   none
#
# return:  none
#
# 01/13/03  46981  MSwam    first version
#---------------------------------------------------------------------------
def CloseTrl ():
  try:
    _trl_fd.close()
  except:
    # failure to close is not an error
    pass

#--------------------------------------------------------------------------
#
# name:    UsingLvl
#
# purpose: States whether the input level will be used or ignored.
#
# input:   level - msg level (see PrintMsg)
#
# return:  True if it will be used, False if not
#
# 12/15/05  54226  Sontag   first version
# 03/26/07  57369  Sontag   This for speed; rm assert - assume syntax is OK
#---------------------------------------------------------------------------
def UsingLvl (level):
   # if level is not active say so
   if level in _active_levels:
      return True
   else:
      return False

#--------------------------------------------------------------------------
#
# name:    PrintMsg
#
# purpose: Prints a datetime-stamped string to STDOUT, along with a
#          severity-level indicator.
#
# input:   level - msg level (e.g. D,I,W,E,F for diag,info,warning,error,fatal)
#          msg   - text string to print
#          modulename - optional name of module text string to print
#
# return:  none
#
# 01/13/03  46981  MSwam    first version
# 10/21/04  52107  Sherbert On a hint from Jim Hare: add stdout flush so that
#                           messages are closer to where erros actually happen
# 01/20/05  52348  Sherbert Change "DEGUG" to "DIAG" to be consistent
# 12/15/05  54226  Sontag   Shift some logic to UsingLvl()
#---------------------------------------------------------------------------
def PrintMsg (level, msg, module_name=""):
    # if level is not active, return
    if not UsingLvl(level):
      return

    # get the current date/time and format the message
    current_time = time.strftime("%Y%j%H%M%S", time.localtime(time.time()))

    if level == "W":
      leveltxt = "WARNING"
    elif level == "E":
      leveltxt = "ERROR"
    elif level == "F":
      leveltxt = "FATAL"
    elif level == "D":
      leveltxt = "DIAG"
    else:
      leveltxt = "INFO"

    # Want a consistent format if module_name is or isn't provided
    if len(module_name) != 0:
      module_name = module_name + '-'

    # write to stdout and the trailer, if opened
    print current_time+'-'+level+'-'+leveltxt+'-'+module_name+msg
    if _trl_fd:
      try:
        _trl_fd.write(current_time+'-'+level+'-'+leveltxt+'-'+module_name+msg+'\n')
      except:
        # failed trailer writes are not considered errors
        pass
    #
    sys.stdout.flush()
    return


#--------------------------------------------------------------------------
#
# name:    PrintStr
#
# purpose: Print input to trailer AND STDOUT w/o the normal formatting.  
#          When the results of PrintMsg are returned in an array, using 
#          PrintMsg to get them to STDOUT and the TRL causes extra 
#          time-stamps and severities to be written.  Writing the output 
#          to STDOUT is easy, this method allows me to get it into the 
#          trailer, too, without changing the time-stamp on original
#          messages.
#
#  For example, let us say the asn_product_status.pl output were the list: 
# ['2009009182958-I-INFO---- start --- Assoc Prod Stat Updater ------',
#  '2009009182958-I-INFO---- ASN: n3uy01080_asn.fits ----'] 
#
# If we loop through the list and send each item through PrintMsg, we get: 
# 2009009183001-I-INFO-2009009182958-I-INFO---- start --- Assoc Prod Stat Updater -----
# 2009009183001-I-INFO-2009009182958-I-INFO---- ASN: n3uy01080_asn.fits ----
# etc.
# 
# If we cut off the first 20 or so chars, we get a misleading time-stamp:
# 2009009183001-I-INFO---- start --- Association Product Status Updater -----
# 2009009183001-I-INFO---- ASN: n3uy01080_asn.fits ----
#
# I intend for this method to give us the sensible thing in both STDOUT and trailer:
# 2009009182958-I-INFO---- start --- Assoc Prod Stat Updater -----
# 2009009182958-I-INFO---- ASN: n3uy01080_asn.fits ----
#
# input:   msg   - text string to print
#
# return:  none
#
# 01/09/09  52348  Sherbert first attempt
#---------------------------------------------------------------------------
def PrintStr( msg ):

    # write to stdout and the trailer, if opened
    print msg
    if _trl_fd:
      try:
        _trl_fd.write(msg+'\n')
      except:
        # failed trailer writes are not considered errors
        pass
    #
    sys.stdout.flush()
    return


#--------------------------------------------------------------------------
#
# name:    RemoveIfThere
#
# purpose: Removes a file from  disk, if it is there.
#          No error or warning is given if the file is not present.
#          Other errors, like an illegal filespec, result in an
#          exception.
#
# input:   filename
#
# return:  none
#
# 01/13/03  46981  MSwam    first version
#---------------------------------------------------------------------------
def RemoveIfThere (filename):

    if os.path.exists(filename):
       os.remove(filename)

    return

#--------------------------------------------------------------------------
#
# name:    StretchFile
#
# purpose: Locates a file on disk given a filename with a
#          VMS-style "stretched" variable name, if one exists.
#          (e.g. OPUS_DEFINITIONS_DIR:my.path)
#
# description:  Uses the OPUS tool "osfile_stretch_file" by issuing
#               a subprocess to invoke the tool.  If the stretched
#               filename argument given resolves to a disk file,
#               the name of the disk file is returned.  If not, the
#               input stretched filename input is returned as the output.
#
# input:   stretched_filename
#
# return:  resolved filename or an echo of the input filename
#
# 01/13/03  46981  MSwam    first version
# 09/16/10  65786  fpierfed Aliased StretchFile with osfile_stretch.findOne
#---------------------------------------------------------------------------
StretchFile = osfile_stretch.findOne

#--------------------------------------------------------------------------
#
# name:    FileToList
#
# purpose: Reads a file and converts each line to a list entry.
#
# input:   filename - name of file to read
#
# return:  list of tokens from file
#
# 02/13/03  xxxxx  MSwam    first version
#---------------------------------------------------------------------------
def FileToList (filename):
  theList = []
  for line in open(filename, 'r').readlines():
    # trim newline and save in list
    theList.append(line[:-1])
  #
  return theList

#--------------------------------------------------------------------------
#
# name:    ResourceToMap
#
# purpose: Reads a file of keyword = value lines and converts each line to a
#          map entry keyed on "keyword".  Ignores lines that start with !
#
# input:   filename - name of file to read
#
# return:  map of tokens from file
#
# 01/15/04  49551  MSwam    first version
#---------------------------------------------------------------------------
def ResourceToMap (filename):
  theMap = {}
  for line in open(filename, 'r').readlines():
    # trim comment (find ! and remove from there until EOS)
    # and if line now empty, read next line
    theLine = line.split('!')
    if len(theLine[0]) == 0:
      continue
    # work with theLine[0] now
    # parse "keyword = value" into a map entry by splitting on =
    # then trimming leading/trailing blanks and newline from parts
    parts = theLine[0].split("=")
    if len(parts) > 1:
      # non-blank line, use it
      key = parts[0].strip()
      value = parts[1].strip()
      theMap[key] = value
  #
  return theMap


def ListToFile(list, filename):
  """Writes a list of items to a file, one item per line

    History:
    -------- -----  --------  --------------------------------------
    10/26/06 54989  MSwam     first version


  """
  out_fd = open(filename, 'w')
  for item in list:
      # convert item to string and add a newline
      out_fd.write(str(item)+ '\n')
  out_fd.close()



## 09 Jan 2009
## Copying these useful utilities directly from auto/test/tools/PipelineUtils.py
#
# A simple wrapper to popen to avoid duplication of these steps each time 
# a system command is run.  Puts the output of the command (stdout&stderr)
# in the return string.
#
def getOutputOf(cmd):
   # run the command with popen
   cmdPipes = os.popen4(cmd) # use popen4 to grab both stdout and stderr
   r0 = cmdPipes[0].close()
   cmdOut = cmdPipes[1].read()
   r1 = cmdPipes[1].close()
#  args.append(r1) # unfortunately, exit status isn't getting set as r1 or r2 !
   if (len(cmdOut) > 0 and cmdOut[-1] == '\n'):
      cmdOut = cmdOut[0:-1]
   return cmdOut

# Same as above, but return as a list of lines
def getListOutputOf(cmd):
   buf = getOutputOf(cmd)
   theList = buf.split("\n")
   while (theList.count('') > 0): theList.remove('')
   return theList 

# Same as above, but only get one line of stdout so pipe does not block
def getSingleLineOutputOf(cmd):
   thePipe = os.popen(cmd)
   cmdOut = thePipe.readline()
   thePipe.close()
   if (len(cmdOut) > 0 and cmdOut[-1] == '\n'):
      cmdOut = cmdOut[0:-1]
   return cmdOut


#--------------------------------------------------------------------------
#
# name:    DateTrim
#
# purpose: Trims off the subseconds from a date string, since reading a
#          date value from a MSSQL database, and then trying to use that
#          same date value in an insert/update generates a format conversion
#          error.
#
# input:   datestr - input date, possibly containing sub-seconds
#
# return:  input date str with sub-seconds truncated (if they were present)
#
# 11/02/10  xxxxx  MSwam    first version
#---------------------------------------------------------------------------
def DateTrim(datestr):
  #
  # the subseconds will be digits following the final "."
  # e.g. 10/14/2010 14:32:55.012023
  #
  # apply the regex and if no subseconds were present, just return unchanged
  datematch = SUBSECONDS_MATCH.search(datestr)
  if datematch == None:
     PrintMsg("D","No subseconds found for: "+datestr)
     return datestr

  # the subseconds-less part is in group 1
  return datematch.group(1)


#========================================================================
# TEST
# % python opusutil.py
#========================================================================
if __name__ == "__main__":
  fname="TestTrailer.tst"
  OpenTrl(fname)
  print "################################"
  PrintMsg("F","test msg  1")
  PrintMsg("F","test msg  2",__name__)
  PrintMsg("E","test msg  3")
  PrintMsg("E","test msg  4",__name__)
  PrintMsg("W","test msg  5")
  PrintMsg("W","test msg  6",__name__)
  PrintMsg("I","test msg  7")
  PrintMsg("I","test msg  8",__name__)
  PrintMsg("D","test msg  9")
  PrintMsg("D","test msg 10",__name__)
  CloseTrl()
  print "================================"
  print "now here is the trailer content:"
  print "==========Beg trl==============="
  for line in open(fname, 'r').readlines():
    print line,
  print "==========End trl==============="
  aList = FileToList(fname)
  print "after reading to a list:"+str(aList)
  RemoveIfThere(fname)
  print "stretching OPUS_DEFINITIONS_DIR:null.path = "+StretchFile ("OPUS_DEFINITIONS_DIR:null.path")
  datestr = "11/01/2010 12:22:52.41941414"
  print "datestr with subseconds      : "+datestr
  print "datestr trimmed of subseconds: "+DateTrim(datestr)
  print "datestr with no subseconds   : "+DateTrim("Jan 01 2010 12:44:55")
