#ifndef EX_ASSERT_H_DEFINED
#define EX_ASSERT_H_DEFINED

//////////////////////////////////////////////////////////////////////////////
//
// Name:
//
//    Assert  (reused from SHARE-OPUS)
//
// Purpose:
//
//    Provide a C++ assert capability which throws an exception.
//    In the case of an incorrect assertion, a std::logic_error is thrown
//    with the given text.
//
//    The great advantage to this being a macro instead of a function, is
//    that, due to C/C++ short-circuiting, the second argument is not
//    evaluated - and thus any string-manip funtions it uses are not
//    called - unless the assertion fails.  If this were a function call,
//    all arguments would be evaluated all the time, being a possibly huge
//    performance hit in applications which liberally use assertions.
//
//    For this reason, it is suggested that these assertions never be turned
//    off (as there is little/no performance gain).  The code to disable these
//    assertions is commented out.  If this decision is ever changed, it 
//    should at most be made switchable with the default to be ON, since
//    some subsystems (e.g. kepler) rely on these being always turned ON.
//
// Example:
//
//    Assert( (ptr != 0), "Unexpected zero pointer!" + createStackTraceEtc() );
//
// Modification History:
//
// Date      PR       Who         Reason
// --------  -------- ----------  --------------------------------------------
// 05/05/06  54226    Sontag      Initial version.
// 10/01/06  52073    Sontag      Renamed include file to avoid conflicts
//
//////////////////////////////////////////////////////////////////////////////

#include <stdexcept>

//#ifdef NO_EX_ASSERTS
  // some define no-op as "((void)0)" but do/while enforces the final semicolon
//#define Assert(unused1, unused2) do {} while( false )
//#else
// If the assertion is false, throw a std::logic_error exception
  #define Assert(e, s) ((e) ? ((void)0) : (throw std::logic_error(s)) )
//#endif

#endif /* EX_ASSERT_H_DEFINED */
