#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "ostdb.h"
#include "loadfile.h"
#include "loadutils.h"
#include "cdbs_sql_gen.h"

/*
******************************************************************************
*
* Function:     get_old_equiv_class
*
* Purpose:      Gets the old equivalence class information based on the 
*               comparison file and the mode keywords
*     
*               The following database query is performed:
*
*               SELECT equivalence_class_severe, equivalence_class_moderate
*               FROM <instrument>_row
*               WHERE
*               file_name = <comparison_file_name>
*               AND
*               <mode_keyword1> = <mode_value1>
*               AND
*               <mode_keyword2> = <mode_value2>
*               ...
*               AND
*               <mode_keywordN> = <mode_valueN>
*
* Returns:      a pointer to an EquivClass structure containing the old 
*               equivalence class info if successful
*               NULL if not successful 
*
******************************************************************************
*/

EquivClass *get_old_equiv_class(
			      table *load_table,   /* i: load table          */
		              int    row,          /* i: row in load table   */
			      char  *row_table,    /* i: database table name */
			      char  *compare_file, /* i: comparison file name*/
		              KeyList *mode_keys[], /* i: structure containing 
  			                                 keyword names       */
		              int    num_mode,     /* i: # of mode keywords  */
			      int   *match         /* o: TRUE if a matching
						         row is found        */
			      )
{
  char temp[1024];
  int j, sev, mod, rec_count;
  *match = TRUE;

  /*
   * Make sure that an entry exists in the database for the comparison file.
   * If not, it is an error.
   */
  ostdb_query_setup(STDB_CONN_1, "get_old_equiv_class", 1, STDB_SQL, 
		    "CDBS_SERVER", "CDBS_DB", NULL);

  stdb_bind_int(STDB_CONN_1, "rec_count", &rec_count);

  sprintf(temp, "SELECT count(*) rec_count from %s", row_table);
  stdb_query(STDB_CONN_1, temp); 
  sprintf(temp, "WHERE file_name = '%s'", compare_file);
  stdb_query(STDB_CONN_1, temp);

  while(ostdb_get_exec(STDB_CONN_1) == STDB_CONTINUE){
  }

  if (rec_count < 1){
    printf("Error: the comparison file %s doesn't exist in the database\n",
	   compare_file);
    stdb_close_query(STDB_CONN_1);
    return(NULL);
  }

  stdb_close_query(STDB_CONN_1);

  /*
   * Query the database for the equivalence class info
   */
  ostdb_begin_transaction(STDB_CONN_1, "CDBS_SERVER", "CDBS_DB");
  ostdb_query_setup(STDB_CONN_1, "get_old_equiv_class", 2, STDB_SQL, 
		    "CDBS_SERVER", "CDBS_DB", NULL);

  stdb_bind_int(STDB_CONN_1, "equivalence_class_severe", &sev);
  stdb_bind_int(STDB_CONN_1, "equivalence_class_moderate", &mod);

  stdb_query(STDB_CONN_1, "SELECT equivalence_class_severe,");
  stdb_query(STDB_CONN_1, "equivalence_class_moderate FROM");
  sprintf(temp, "%s WHERE file_name='%s'", row_table, compare_file);
  stdb_query(STDB_CONN_1, temp);

  /*
   * Add mode keywords to the query
   */
  for (j=0; j<num_mode; j++){
    if (mode_keys[j]->quoted){
      sprintf(temp, "and %s='%s'", mode_keys[j]->db_name, 
	      UpCase(get_tab_cell(load_table, mode_keys[j]->colname, row))); 
    }else{
      sprintf(temp, "and %s=%s", mode_keys[j]->db_name, 
	      UpCase(get_tab_cell(load_table, mode_keys[j]->colname, row))); 
    }
    stdb_query(STDB_CONN_1, temp);
  }
  
  while(ostdb_get_exec(STDB_CONN_1) == STDB_CONTINUE){
  }

  rec_count = stdb_record_count(STDB_CONN_1);

  stdb_close_query(STDB_CONN_1);

  /*
   * If there was an error with the query - rollback
   */      
  if (ostdb_get_status(STDB_CONN_1) != OSTDB_NORMAL){
    ostdb_abort_transaction(STDB_CONN_1);
    return(NULL);
  }

  ostdb_end_transaction(STDB_CONN_1);

  EquivClass *old_equiv=NULL;
  old_equiv = (EquivClass *)calloc(1, sizeof(EquivClass));

  if (rec_count < 1){ 
    //
    // The comparison file has no entry for the given modes.
    // Just return, the new equivalence class values will be used
    //
    printf("\nWarning: No comparison file records matched mode values ");
    printf("for row %-d.\n         New equivalence class values and", row+1);
    printf(" a SEVERE change_level were used\n");
    *match = FALSE;

  }else{ /* We did find a matching record */ 

    if((mod == -1) || (sev == -1)){

      //
      // The comparison file has no evquivalence class values for the given 
      // modes. We need to get the values from the equiv_class_value table
      // and update the comparison file record in the database 
      // then return those same values for this file.
      //
      old_equiv = get_new_equiv_class();
      
      //
      // Update the comparison file record with the new equiv class values
      //
      ostdb_begin_transaction(STDB_CONN_1, "CDBS_SERVER", "CDBS_DB");
      ostdb_query_setup(STDB_CONN_1, "get_old_equiv_class", 3, STDB_SQL, 
			"CDBS_SERVER", "CDBS_DB", NULL);
      sprintf(temp, "update %s set equivalence_class_severe = %-d,", 
	      row_table, old_equiv->sev);
      stdb_query(STDB_CONN_1, temp);
      sprintf(temp, "equivalence_class_moderate = %-d where file_name='%s'",
	      old_equiv->mod, compare_file);
      stdb_query(STDB_CONN_1, temp);
      
      //
      // Add mode keywords to query
      //
      for (j=0; j<num_mode; j++){
	if (mode_keys[j]->quoted){
	  sprintf(temp, "and %s='%s'", mode_keys[j]->db_name, 
		  UpCase(get_tab_cell(load_table, mode_keys[j]->colname, 
					    row))); 
	}else{
	  sprintf(temp, "and %s=%s", mode_keys[j]->db_name, 
		  UpCase(get_tab_cell(load_table, mode_keys[j]->colname, 
					    row))); 
	}
	stdb_query(STDB_CONN_1, temp);
      }
      
      while(ostdb_get_exec(STDB_CONN_1) == STDB_CONTINUE){
      }
      
      rec_count = stdb_record_count(STDB_CONN_1);
      
      stdb_close_query(STDB_CONN_1);
      
      //
      // If there was an error with the query - rollback
      //      
      if (ostdb_get_status(STDB_CONN_1) != OSTDB_NORMAL) {
	ostdb_abort_transaction(STDB_CONN_1);
	// LEAK: free(old_equiv); old_equiv=NULL;
	return(NULL);
      }else{
	ostdb_end_transaction(STDB_CONN_1);
	printf("\nWarning: The matching comparison file record has no ");
	printf("equivalence class values\n         for row %-d.", row+1);
	printf(" New equivalence class values were used\n");
      }
    }else{   /* there is a matching entry */
      old_equiv->mod = mod;
      old_equiv->sev = sev;
    }
  }
  return (old_equiv);
}
