#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "ostdb.h"
#include "loadfile.h"
#include "loadutils.h"
#include "cdbs_sql_gen.h"

/*
******************************************************************************
*
* Function:     load_expanded
*
* Purpose:      Generates SQL to load the contents of an expanded load file 
*               into the database. 
*
*               For each expansion number, there is a row-level
*               INSERT statement.
*
*               The row level insert statement is of the form:
*               INSERT  <instrument>_row (file_name,
*                                         expansion_number,
*                                         <mode_keyword1>,
*                                         <mode_keyword2>,
*                                         ...
*                                         <mode_keywordN>,
*                                         equivalence_class_severe,
*                                         equivalence_class_moderate,
*                                         pedigree,
*                                         observation_begin_date,
*                                         observation_end_date,
*                                         comment)
*                                 VALUES (<file_name>,
*                                         <expansion_number>,
*                                         <mode_value1>,
*                                         <mode_value2>,
*                                         ...
*                                         <mode_valueN>,
*                                         <equivalence_class_severe>,
*                                         <equivalence_class_moderate>,
*                                         <pedigree>,
*                                         <observation_begin_date>,
*                                         <observation_end_date>,
*                                         <comment>)
*
*               There will also be a file-level insert statement generated
*               for each expansion number of the form:
*
*               INSERT  <instrument>_file (delivery_number,
*                                          reference_file_type,
*                                          file_name,
*                                          expansion_number,
*                                          useafter_date,
*                                          comparison_file_name,
*                                          opus_flag,
*                                          comment,
*                                          reject_flag)
*                                  VALUES (delivery_num,
*                                          <reference_file_type>,
*                                          <file_name>,
*                                          <expansion_number>,
*                                          <useafter_date>,
*                                          <comparison_file>,
*                                          <opus_flag>,
*                                          <comment>,
*                                          'N')
*
*               Values in <> come from the load file, others are determined 
*               in this software.
*
*
* Returns:      OK or ERROR
*
******************************************************************************
*/

int load_expanded(
			 table *load_table, /* i: table containing load data */
			 FILE *sql_file,    /* i: pointer to SQL file        */
			 int delivery       /* i: delivery number            */
			){

  char *file_name, *file_type, *instrument, *use_after, *opus_flag;
  char *compare_file, *comment, *colname, *colval; 
  char * expansion_num, *pedigree, *begin_date, *end_date, *row_comment;  
  int num_mode, i, row, col;
  KeyList *mode_keys[MAXMODE];
  char row_table[MAX_NAME];
  char file_table[MAX_NAME];
  char *change_level;
  EquivClass *old_equiv = NULL, *new_equiv = NULL, *equiv_class = NULL;
  int match;
  char *quote_fixup=NULL;

  match = TRUE;

  /* 
   * Get info from load file header
   */
  file_name = UpCase(get_tab_key(load_table, "FILE_NAME"));
  file_type = UpCase(get_tab_key(load_table, "REFERENCE_FILE_TYPE"));
  instrument = UpCase(get_tab_key(load_table, "INSTRUMENT"));
  use_after = UpCase(get_tab_key(load_table, "USEAFTER_DATE"));
  opus_flag = UpCase(get_tab_key(load_table, "OPUS_FLAG"));
  compare_file = UpCase(get_tab_key(load_table, "COMPARISON_FILE"));
  comment = get_tab_key(load_table, "COMMENT");
  if (isspace(comment[0]) && !isspace(comment[1])){
    comment++;
  }

  /*
   * Determine table names
   */
  sprintf(file_table, "%s_file", instrument);
  LowCase(file_table);
  sprintf(row_table, "%s_row", instrument);
  LowCase(row_table);

  /* 
   * Determine column names for mode keywords
   */
  num_mode = get_modes(instrument, file_type, mode_keys);

  /* 
   * Do row level inserts
   */
  for (row=0; row<nrow_table(load_table); row++){
    /*
     * get row level fixed keyword values
     */
    pedigree = UpCase(get_tab_cell(load_table, "PEDIGREE", row)); 
    begin_date = UpCase(get_tab_cell(load_table, "OBSERVATION_BEGIN_DATE", 
				     row)); 
    end_date = UpCase(get_tab_cell(load_table, "OBSERVATION_END_DATE", row)); 
    row_comment = get_tab_cell(load_table, "COMMENT", row);
    if (isspace(row_comment[0]) && !isspace(row_comment[1])){
      row_comment++;
    }
    expansion_num = UpCase(get_tab_cell(load_table, "EXPANSION_NUMBER", row));

    /*
     * Determine equivalence class info
     */
    if (strcmp(compare_file, "(INITIAL)") != 0){
      if ((old_equiv = get_old_equiv_class(load_table, row, row_table, 
					   compare_file, mode_keys, 
					   num_mode, &match)) == NULL){
	printf("Error obtaining comparison file equivalence class values\n");
	return(ERROR);
      }
    }
    
    if ((new_equiv = get_new_equiv_class()) == NULL){
      printf("Error obtaining new equivalence class values\n");
      return(ERROR);
    }
    
    /*
     * Assign new equivalence class values based on severity
     */
    if (match){
      change_level = UpCase(get_tab_cell(load_table, "CHANGE_LEVEL", row));
      equiv_class = set_equiv_class(change_level, compare_file, old_equiv, 
				    new_equiv);
    }else{
      equiv_class = set_equiv_class("SEVERE", compare_file, old_equiv, 
				    new_equiv); 
    }

    if (equiv_class == NULL){
      printf("An ERROR occured while determining equivalence class info\n");
      return(ERROR);
    }

    if (new_equiv != NULL){
      // ANOTHER case of free failing; just leak and go on
      // free(new_equiv); 
      new_equiv=NULL;
    }
    if (old_equiv != NULL){
      // LEAK: free(old_equiv); 
      old_equiv=NULL;
    }

    /*
     * Do INSERT part
     */
    fprintf(sql_file, "INSERT %s\n\t( file_name", row_table);
    fprintf(sql_file, ",\n\t expansion_number");
    fprintf(sql_file, ",\n\t delivery_number");
    
    for (i=0; i<num_mode; i++){
	if((colval = (get_tab_cell(load_table, mode_keys[i]->colname, row)))
	   != NULL){
	  if (colval[0] != '\0'){
	    fprintf(sql_file, ",\n\t %s", mode_keys[i]->db_name);
	  }
	}else{
	  printf("Mode keyword: %s is missing from the load file\n",
		 mode_keys[i]->colname);
	  return ERROR;
	}
    }
    
    fprintf(sql_file, ",\n\t equivalence_class_severe");
    fprintf(sql_file, ",\n\t equivalence_class_moderate");


    if ((pedigree != NULL) && (pedigree[0] != '\0')){
      fprintf(sql_file, ",\n\t pedigree");
    } 
    if ((begin_date != NULL) && (begin_date[0] != '\0')){
      fprintf(sql_file, ",\n\t observation_begin_date");
    }
    if ((end_date != NULL) && (end_date[0] != '\0')){
      fprintf(sql_file, ",\n\t observation_end_date");
    }

    if (NotBlank(row_comment)){
      fprintf(sql_file, ",\n\t comment");
    }
    
    /*
     * Do VALUES part
     */
    fprintf(sql_file, ")\nVALUES ('%s'",file_name);
    fprintf(sql_file, ",\n\t%s", expansion_num); 
    fprintf(sql_file, ",\n\t%-1d", delivery); 

    for (i=0; i<num_mode; i++){
      colval = UpCase(get_tab_cell(load_table, mode_keys[i]->colname, row));
      if (colval[0] != '\0'){
	if (mode_keys[i]->quoted){
	  fprintf(sql_file, ",\n\t'%s'", colval); 
	}else{
	  fprintf(sql_file, ",\n\t%s", colval); 
	  }
      }
    }

    fprintf(sql_file, ",\n\t%-1d", equiv_class->sev);
    fprintf(sql_file, ",\n\t%-1d", equiv_class->mod);

    if ((pedigree != NULL) && (pedigree[0] != '\0')){
      fprintf(sql_file, ",\n\t'%s'", pedigree);
    } 
    if ((begin_date != NULL) && (begin_date[0] != '\0')){
      fprintf(sql_file, ",\n\t'%s'", begin_date); 
    }
    if ((end_date != NULL) && (end_date[0] != '\0')){
      fprintf(sql_file, ",\n\t'%s'", end_date); 
    }

    if (NotBlank(row_comment)){
      TransformQuotes(row_comment, &quote_fixup);
      fprintf(sql_file, ",\n\t\'%s\'",quote_fixup); 
      if (quote_fixup) {
         // freeing here was causing crashes sometimes; just leak instead
         // free(quote_fixup);
         quote_fixup=NULL;
      }
    }
    fprintf(sql_file, ")\ngo\n");
    
    /*
     * Write out SQL to insert file-level record
     */

    /*
     * Do INSERT part
     */
    fprintf(sql_file, "INSERT %s\n\t", file_table);
    fprintf(sql_file, "(delivery_number,\n\t");
    fprintf(sql_file, "reference_file_type,\n\t");
    fprintf(sql_file, "file_name,\n\t");
    fprintf(sql_file, "expansion_number,\n\t");
    fprintf(sql_file, "useafter_date,\n\t");
    fprintf(sql_file, "comparison_file_name,\n\t");
    fprintf(sql_file, "opus_flag,\n\t");
    fprintf(sql_file, "comment,\n\t");
    fprintf(sql_file, "reject_flag)\n");    

    /*
     * Do VALUES part
     */
    fprintf(sql_file, "VALUES (%-1d,\n\t", delivery);
    fprintf(sql_file, "'%s',\n\t", file_type);
    fprintf(sql_file, "'%s',\n\t", file_name);
    fprintf(sql_file, "%s,\n\t", expansion_num);
    fprintf(sql_file, "'%s',\n\t", use_after);
    fprintf(sql_file, "'%s',\n\t", compare_file);
    if((opus_flag == NULL) || (opus_flag[0] == '\0')){
      fprintf(sql_file, "'N',\n\t");
    }else{
      fprintf(sql_file, "'%s',\n\t", opus_flag);
    }
    TransformQuotes(comment, &quote_fixup);
    fprintf(sql_file, "\'%s\',\n\t", quote_fixup);
    if (quote_fixup) {
       // LEAK: free(quote_fixup);
       quote_fixup=NULL;
    }
    fprintf(sql_file, "'N')\ngo\n");       /* reject flag           */
  }
  return(OK);
}

