#include <stdio.h>
#include "error.h"
#include "config.h"
#include "cdbslib.h"
#include "loadfile.h"
#include "reffile.h"
#include "certify.h"
#include "template.h"
#include "cdbserror.h"

//-------------------------------------------------------------------------
// CERTIFY -- Check the information in a reference file or load file
//            against a template of allowed values.
//
// i: catalog        open catalog file descriptor
// i: reffile_name   reference file or load file name
// u: cached_tpl     cached template descriptor
//
// History:
// ??/??/??  xxxxx  BSimon    original
// 03/28/11  67644  MSwam     reworked for Linux/MSSQL
//
//-------------------------------------------------------------------------
void certify (table *catalog, char *reffile_name, tpl_file *cached_tpl)
{
    // Print message identifying the file
    char message[SZ_LINE+1];
    sprintf (message, "== Checking %s ==\n", reffile_name);
    cdbs_message (INFO_MSG, message);

    // Locate the catalog entry for this file
    int row;
    if ((row = find_lookup (catalog, reffile_name)) == ERR) {
	cdbs_warn (NOLOOK, "%s", reffile_name);
	return;
    }

    // Retrieve the template file basename and the file type for this
    //   file from the matching catalog row
    //
    char *tplbase_name = get_tab_cell (catalog, "tplfile", row);
    char *reffile_type = get_tab_cell (catalog, "filetype", row);

    // Form the full template pathname from the catalog's directory and 
    //   the template file basename
    char tplfile_name[SZ_FNAME+1];
    pathname (tplfile_name, name_lookup (catalog), tplbase_name, SZ_FNAME);
	
    // Open the template file. 
    // (It is cached in cached_tpl, so it is not opened repeatedly)
    //
    tpl_file *tpl = open_template (tplfile_name, cached_tpl);
    cached_tpl = tpl;

    // Open the file HEADER (FITS primaryHDU or 1st loadfile keyword group)
    reffile *ref = open_reffile_hdr (reffile_name, reffile_type, "r");
    if (ref == NULL) {
	cdbs_warn (NOTOPEN, "%s", reffile_name);
	return;
    }
    // Check any header keywords specified in the template,
    //    and optimize body keywords for faster checking later
    //
    tpl_record	*rule = NULL;
    while ((rule = next_template (tpl, rule)) != NULL) {
	if (body_rule (rule)) {
            // only optimize body keywords, don't evaluate yet
	    study_rule (rule);
	} else {
            // verify a header keyword against the template rule
	    check_rule (rule, ref, 0);
	}
    }

    // Open the file BODY (FITS 1st extension or later loadfile keyword groups)
    ref = open_reffile_body (reffile_name, reffile_type, "r");
    if (ref == NULL) {
	cdbs_warn (NOTOPEN, "%s", reffile_name);
	return;
    }
    // Check body keywords. Take exit if no keywords left to check 
    int irow = 0;
    int nrow = 0;
    while (skip_template (tpl) == NO) {
        // Nrow is only evaluated conditionally and once
        // because it is an expensive operation for fits images
	//
        if (nrow == 0)
	    nrow = nrow_reffile (ref);

	if (irow >= nrow)
	    break;

	rule = NULL;
	while ((rule = next_template (tpl, rule)) != NULL)
	    check_rule (rule, ref, irow);

	irow ++;
    }
    close_reffile (ref);
}
