//==========================================================================
// DATA STRUCTURE LIBRARY
//
// Name: lookup.cpp
//
// Purpose: 
//   This set of routines is used to read the cdbscatalog.dat file 
//   content, and to search for various pieces of information within it.
//   The file is called a "lookup" file, but these routines are NOT generic
//   and require a very specific file format that is likely only met by
//   the cdbscatalog.dat file.
//
// Description:
//   The file content is read into a table data structure 
//   (see libs/lf_table.cpp and inc/loadfile.h) by the open_lookup() call.
//   The data structure can then be searched by the find_lookup() call.
//
// Functions:
//   close_lookup - frees the memory used by a catalog table
//   find_lookup - locates information in the catalog table for a ref file
//   name_lookup - returns the filename of the catalog file from the table hdr
//   open_lookup - reads the catalog file content into a table structure
//
// History:
//   ??/??/???? BSimon   original
//   03/10/2011 MSwam    add comments and rename a few items for clarity
//
//==========================================================================
#include <stdio.h>
#include <string.h>
#include "config.h"
#include "error.h"
#include "cdbslib.h"
#include "reffile.h"
#include "loadfile.h"
#include "cdbserror.h"

#define  NKEY		4   // max number of allowed reference file-specific 
                            // key=val pairs in a table row
#define  SZ_KEYVAL	63
#define  MAXFIELD	10  // max number of columns in the catalog table
#define  NPAIR		2   // a pair of anything is always TWO (!)

// These values are used to indicate what phase of the table search 
// is in progress. We start with ZIP (none), and move on to the file
// extension (EXT), the file type (TYPE), and any reference-file-specific
// key values (KEY).
//
#define  MATCH_ZIP	0
#define  MATCH_EXT	1
#define  MATCH_TYPE	2
#define  MATCH_KEY	3

// These are the names of the table columns used INTERNALLY by the code.
// (they do NOT appear in the catalog file anywhere).  The first 3 are fixed
// (filename, extension, filetype), and the next group are variable,
// depending on what key=val pairs are included in the table row for the 
// particular reference file being described.
//
static char	filename_key[] = "filename";
static char	extname_key[]  = "extension"; // usually .fits or .lod
static char	typenam_key[]  = "filetype";  // usually image, table, or load
static char	*keyname[] = {"keyname1", "keyname2", "keyname3", "keyname4"};
static char	*valname[] = {"keyval1", "keyval2", "keyval3", "keyval4"};

//------------------------------------------------------------------------
// CLOSE_LOOKUP -- Free the resources used by a catalog table
//
//------------------------------------------------------------------------
void close_lookup (table *catalog) // i - catalog table pointer
{
    free_table (catalog);
}

//------------------------------------------------------------------------
// FIND_LOOKUP -- Search a table for the row matching a reference file
//
// History
// -------
// 06/12/02  45993  mss  add error messages
// 03/10/11  xxxxx  mss  add comments and type_reffile() call
//------------------------------------------------------------------------
int find_lookup (table *catalog, // i - catalog table structure pointer
                 char *fname)    // i - reference filename whose file extension
                                 //     will be used to search the table
{
    // verify the existence of the reference file on disk
    //
    FILE *fd;
    if ((fd = fopen (fname, "r")) == NULL) {
        cdbs_warn (NOTOPEN, "%s", fname);
	return (ERR); 
    }
    fclose (fd);
    cdbs_debug("OK: file %s exists...",fname);

    // determine the file type from the file content (image, table, or loadfile)
    //   (will be used to match catalog row entries)
    //
    char actual_filetype[30];
    type_reffile(fname, actual_filetype);

    reffile *ref = NULL;
    ref = open_reffile_hdr (fname, actual_filetype, "r");
    if (ref == NULL) {
        fprintf(stderr,"Failed to open %s as filetype=%s\n",
                        fname, actual_filetype);
        cdbs_warn (NOTOPEN, "%s", fname);
	return (ERR);
    }

    // Get the column numbers of the columns used to search the table.
    // The extension and file type columns are always used.
    // Then there are variable number of key=value pair columns, depending
    //   on the information in the table row for the particular ref file.
    //
    int extcol, typecol, keycol[NKEY], valcol[NKEY];
    if ((extcol = get_tab_colnum (catalog, extname_key)) == BAD)
	cdbs_error (NOCOL, "%s", extname_key);

    if ((typecol = get_tab_colnum (catalog, typenam_key)) == BAD)
	cdbs_error (NOCOL, "%s", typenam_key);

    int ikey;
    for (ikey = 0; ikey < NKEY; ikey ++) {
	if ((keycol[ikey] = get_tab_colnum (catalog, keyname[ikey])) == BAD)
	    cdbs_error (NOCOL, "%s", keyname[ikey]);

	if ((valcol[ikey] = get_tab_colnum (catalog, valname[ikey])) == BAD)
	    cdbs_error (NOCOL, "%s", valname[ikey]);
    }

    // Search for the row which matches the reference file content.
    //
    lod_error = NULL;    // this is a GLOBAL variable in loadfile.h

    int search_phase = MATCH_ZIP; // This is a flag that is used to indicate what
                              // phase of the table search is in progress.
                              // It gets initialized to ZIP (=none), and
                              // changes as each new piece of info is found. 
                              // It is only used if an error occurs, 
                              // so that an error message particular to the
                              // phase that had the problem can be written.

    // grab the filename extension for looking up in the table
    // (in the modern era that will only be .fits or .lod, back in the
    //  GEIS days it was more unique per reference file)
    //
    char dir[SZ_FNAME+1];
    char root[SZ_FNAME+1];
    char filext[SZ_FNAME+1];
    breakfname (fname, dir, root, filext, SZ_FNAME);

    // Search the rows of the catalog table, looking for a match against
    // the reference file's characteristics
    //
    char *entry_filetype = NULL;
    char *lookext, *lookkey, *lookval;
    char fileval[SZ_KEYVAL+1];
    int irow;

    for (irow = 0; irow < nrow_table(catalog); irow ++) {

        // grab the file extension value from the current table row
        //   and compare it to the reference file extension
        //
	lookext = get_tab_cell_bynum (catalog, extcol, irow);
	if (lookext[0] != EOS && patmatch (lookext, filext) == 0)
            // table row did not match the fileext, try next table row
	    continue;
 
        // moving into the file type search phase now, though we will likely
        //   regress back to file ext matching as needed to move forward in
        //   the table
        //
	if (search_phase < MATCH_EXT)
	    search_phase = MATCH_EXT;

        // Now try to match the file type in the table row with
        // the file type of the reference file or lod file itself
        // (usually image, table, or load).
        //
	if (ref == NULL) {
            // get the table row file type value
	    entry_filetype = get_tab_cell_bynum (catalog, typecol, irow);

            // compare to the actual file type
            if (strcmp(entry_filetype, actual_filetype)!=0) {
                // NO MATCH, this table row is not the one for the ref file
                // (try next table row)
                cdbs_debug("find_lookup: %s is not type = %s\n",
                            fname,entry_filetype);
                                continue;
            }
	}
        cdbs_debug("MATCHED: %s is filetype=%s",fname,entry_filetype);

        // moving into the final search phase now (key=value)
	if (search_phase < MATCH_TYPE)
	    search_phase = MATCH_TYPE;

	// Check the header keywords against the table row key values
	for (ikey = 0; ikey < NKEY; ikey ++) {
	    lookkey = get_tab_cell_bynum (catalog, keycol[ikey], irow);
            cdbs_debug("Search for key %s|", lookkey);

	    if (lookkey[0] != EOS) {
	        lookval = get_tab_cell_bynum (catalog, valcol[ikey], irow);

		get_hdr_reffile (ref, lookkey, fileval, SZ_KEYVAL);
		strlcase (fileval);

		if (strcmp (lookval, fileval) != 0) {
                    cdbs_debug("Trying to match %s, but table row=%s|", fileval,lookval);
		   break;
                }
                else {
                    cdbs_debug("table row MATCHED %s|",fileval);
                }
	    }
	}

	if (ikey == NKEY) {
	    search_phase = MATCH_KEY;
	    break;
	}
    }   // end for loop stepping through the catalog table rows

    if (ref != NULL)
        close_reffile (ref);

    // IF we didn't find a match, print a warning message listing
    // the search phase that failed
    // 
    cdbs_debug("DONE. print message depending on search_phase: %d\n",search_phase);
    switch (search_phase) {
    case MATCH_ZIP:
        cdbs_warn (BADEXT, "%s", fname);
        irow = ERR;
        break;
    case MATCH_EXT:
        if (lod_error != NULL) {
	    cdbs_warn (NOLOAD, "%s", lod_error);
	    lod_error = NULL;
	}
        cdbs_warn (BADTYPE, "%s", fname);
        irow = ERR;
        break;
    case MATCH_TYPE:
        cdbs_warn (BADKEY, "%s", fname);
        irow = ERR;
        break;
    case MATCH_KEY:
        break;
    }

    // Return a value of ERR if there was no match.
    // Otherwise return the number of the matched row
    return (irow);
}

//---------------------------------------------------------------------------
// NAME_LOOKUP -- Retrieve the name of the lookup file from the table header
//
//---------------------------------------------------------------------------
char *name_lookup (table *catalog)  // i - table structure pointer
{
    return (get_tab_key (catalog, filename_key));
}

//--------------------------------------------------------------------------
// OPEN_LOOKUP -- Read a catalog file into a table structure
//
//--------------------------------------------------------------------------
table *open_lookup (char *fname,   // i - catalog file name
                    char *columns) // i - list of catalog column names to use
{
    FILE	*fd;
    table 	*catalog;
    int		irow, ikey, ifield, keyfield;
    char	collist[SZ_LINE+1], buf[SZ_FNAME+1];
    char	*colname[MAXFIELD], *field[MAXFIELD];
    char	*keypair[MAXFIELD], *keyval[NPAIR];

    // Open the catalog file 
    fd = fopen (fname, "r");
    if (fd == NULL)
	cdbs_error (NOTOPEN, "%s", fname);

    // create a table data structure and save the filename in its metadata
    catalog = create_table ();
    put_tab_key (catalog, filename_key, fname);

    // Create table columns required for searching 
    add_tab_col (catalog, extname_key);  // file extension
    add_tab_col (catalog, typenam_key);  // file type

    // create table columns for any key=value pairs that might apply 
    // to this ref file
    //
    for (ikey = 0; ikey < NKEY; ikey ++) {
	add_tab_col (catalog, keyname[ikey]);
	add_tab_col (catalog, valname[ikey]);
    }

    // Split the list of column names on commas,
    //   and save the index in the list for the "keyword" field 
    //
    safecopy (collist, columns, SZ_LINE);
    strsplit (',', collist, colname, MAXFIELD);
    for (keyfield = 0; keyfield < MAXFIELD; keyfield ++) {
	if (strcmp (colname[keyfield], "keyword") == 0)
	    break;
    }

    // Read the catalog file rows into the table 
    for (irow = 0; rdlines (fd, buf, SZ_LINE) == OK; irow ++) {
        // split catalog row into fields on whitespace
	strsplit (' ', buf, field, MAXFIELD);

	// Read each field from the line into the table 
	for (ifield = 0; 
	     ifield < MAXFIELD && colname[ifield][0] != EOS; 
	     ifield ++) {

	    if (ifield == keyfield) {
		// The "keyword" field is split into a keyword list
		// and further split into keyword = value pairs.
		// The values are converted to lower case for a case
		// independent match.
                //
		strlcase (field[ifield]);
		strsplit (',', field[ifield], keypair, MAXFIELD);

		for (ikey = 0; ikey < NKEY; ikey ++) {
		    strsplit ('=', keypair[ikey], keyval, NPAIR);
// UPPERCASE the keyword name
strucase(keyval[0]);
		    put_tab_cell (catalog, keyname[ikey], irow, keyval[0]);
		    put_tab_cell (catalog, valname[ikey], irow, keyval[1]);
		}

		if (keypair[ikey][0] != EOS)
		    cdbs_error (MAXKEY, "%s", fname);

	    } else {
		// All other fields are copied as is to the table 

		put_tab_cell (catalog, colname[ifield], irow, field[ifield]);
	    }
	}
    }

    fclose (fd);
    return (catalog);
}
