#include <string.h>
#include "alist.h"
#include "error.h"
#include "config.h"
#include "datafile.h"
#include "cdbslib.h"
#include "cdbserror.h"

//======================================================================
// FILESYSTEM LIBRARY
//   Name: xlate_fname
//======================================================================

#define  MAXITER		8

static alist	*xlate_init (char *);

//-----------------------------------------------------------------------
// XLATE_FNAME -- Translate an iraf virtual file name into an OS file name
//
// i: fname	iraf file name
// o: osfn	operating system file name
// i: maxch	maximum length of os file name
//
// History:
// 09/20/04  52054  MSwam  replace CDBSDIR,DIRECTORY_NAME with topdir call
// 04/08/11  67644  MSwam  replace topdir with datadir
//-----------------------------------------------------------------------
void xlate_fname (char *fname, char *osfn, int maxch)

{
    static alist *dirlist = NULL;

    // build directory map filespec 

    char cdbs_data_dir[SZ_FNAME+1];
    char directory_map_filename[SZ_FNAME+1];
    safecopy(directory_map_filename, datadir(cdbs_data_dir), SZ_FNAME);
    safecat(directory_map_filename, DIRECTORY_NAME, SZ_FNAME);

    // Read the directory map file 
    // on the first pass thru this function

    if (dirlist == NULL)
	dirlist = xlate_init (directory_map_filename);

    // If a filename contains a dollar sign, everything in front of
    // of the dollar sign gets translated by looking up the string 
    // in the directory translation file. This is done repeatedly 
    // until no more translations are found or the maximum number
    // of iterations is exceeded

    int iter = 0;
    char *dollar;
    char *newname;
    char name1[SZ_FNAME+1];
    char name2[SZ_FNAME+1];
    char dir[SZ_FNAME+1];
    char root[SZ_FNAME+1];
    char ext[SZ_FNAME+1];

    safecopy (name1, fname, SZ_FNAME);
    cdbs_debug("xlate_fname: after safecopy: name1,fname = %s,%s", name1,fname);

    while ((dollar = strchr (name1, '$')) != NULL) {
	// Check number of iterations 

	iter ++;
	if (iter > MAXITER)
	    cdbs_error (MAXDEF, "%s", fname);

	// Look up translation 

	*dollar = EOS;
	newname = find_alist (dirlist, name1);
	if (newname == NULL)
	    cdbs_error (NOTRANS, "%s", name1);

	// Copy translation to new string and append everything 
	// following the translation

	safecopy (name2, newname, SZ_FNAME);
	safecat (name2, dollar+1, SZ_FNAME);

	safecopy (name1, name2, SZ_FNAME);
    }

    // If the file name is a relative file name, make it a
    // subdirectory of the top level cdbs directory

    if (name1[0] == '/') {
	safecopy (osfn, name1, maxch);
    } else {
	breakfname (name1, dir, root, ext, SZ_FNAME);
	addsubdir (refdir(cdbs_data_dir), dir, osfn, maxch);
	safecat (osfn, root, maxch);
	safecat (osfn, ext, maxch);
    }
}

// XLATE_INIT -- Read the directory file into an associative list
//
// i: directory_map_filename		directory file name

static alist *xlate_init (char *directory_map_filename)
{
    FILE	*fd;
    alist	*list;
    char	*field[2];
    char	buf[SZ_LINE+1];

    // Open the directory file 

    fd = fopen (directory_map_filename, "r");
    if (fd == NULL)
	cdbs_error (NOTOPEN, "%s", directory_map_filename);

    // Read each line and split it into two fields
    // The first field is the logical name and
    // the second is the translation

    list = create_alist ();
    while (rdlines (fd, buf, SZ_LINE) == OK) {
	strsplit (' ', buf, field, 2);
	if (field[1] != EOS)
	    add_alist (list, field[0], field[1]);
    }

    fclose (fd);
    return (list);
}
