#include <stdlib.h>
#include "config.h"
#include "cdbslib.h"
#include "loadfile.h"
#include "error.h"

//==============================================================
// STRING LIBRARY
//   Name: strmatch
//==============================================================

//---------------------------------------------------------------------------
// Name: strmatch
//
// Description: Find the 1-based index of a "word" string within a 
//              "dictionary" string, it it exists.
//
//              A "dictionary" here is a string of bar-delimited text
//              values (e.g. "one|two|three|").
//
//              The "index" is the 1-based "word"-position within the dictionary
//              of the search word (e.g. word "two" is index position 2 within
//              the example dictionary).
//
// Args:
//   searchWord (char *) - i: input search word
//   dict (char *)       - i: bar-delimited dictionary string
//
// Returns: 1-based integer index into dictionary string of matched word OR
//          0 if no word match was found
//
//---------------------------------------------------------------------------
//
int strmatch (const char *searchWord,  // i: input search word
              const char *dict         // i: dictionary to search within
             )
{
    // Make lower case copy of input word for case-independent match attempt
    char *lc_searchWord = strclone (searchWord);
    strlcase (lc_searchWord);

    int jc = 0;     // search word character index
    int index = 1;  // 1-based word index of current dictionary match candidate
    int state = 0;  // two loop states are defined:
                    //  0=trying to word match 
                    //  1=current word does not match, skipping to next word

    // loop through all the dictionary characters
    for (int ic = 0; dict[ic] != EOS; ic ++) {

	if (lc_searchWord[jc] == EOS && dict[ic] == '|')
            // end of search word and end of dictionary word both reached
            // a MATCH was found
	    break;

	if (dict[ic] == '|') {
	    // end of current dictionary word was reached without a match, 
            // reset counters for next dictionary word comparison
	    jc = 0;
	    index ++;
	    state = 0;

	} else if (state == 0) {
            // see if next dictionary word character matches 
            // next search word character
            //
	    if (dict[ic] == lc_searchWord[jc]) {
                // match candidate still valid, move ahead to next character
		jc ++;
	    } else {
                // match candidate has FAILED, set state to skip ahead to
                // next dictionary word candidate
		jc = 0;
		state = 1;
	    }
	}
    }
    if (lc_searchWord[jc] != EOS)
        // NO MATCH FOUND
	index = 0;

    free (lc_searchWord); lc_searchWord=NULL;
    return (index);
}
