#----------------------------------------------------------------------------
#
# module:  osfile_stretch
#
#         NOTE: Needed so CDBS AUTO tests can run properly.
#               We would rather NOT put this code in the AUTO tree
#               itself, and since CDBS doesn't use SHARE, it gets
#               its own copy to support AUTO tests.
#
# Provides VMS-like "file stretch" capability.  Can be used to find
# a single file in a stretch expression (function findOne)
#
#    e.g. OPUS_DEFINITIONS_DIR:opus_login.csh
#
# or a list of files in a stretch expression with a wildcard (function findAll)
#
#    e.g. OPUS_DEFINITIONS_DIR:*.stage
#
# 10/11/06  56084  MSwam    First version
# 03/24/11  67891  Sherbert replace opusutil with logging
#---------------------------------------------------------------------------
import os
import re
import glob
import logging

STRETCHED_LOGICAL_SEPARATOR = ":"

# finds string-separator-morestring match using a regular expression
STR_SEP_STR_REGEX  = re.compile("(^.+)("+
                               STRETCHED_LOGICAL_SEPARATOR+
                               ")(.+)")  

# OPUS has NONE FATAL    ERROR WARN    INFO DIAG/ALL
# vs python     CRITICAL ERROR WARNING INFO DEBUG
LEVELS = {'MSG_DIAG'    : logging.DEBUG,
          'MSG_ALL'     : logging.DEBUG,
          'MSG_INFO'    : logging.INFO,
          'MSG_WARNING' : logging.WARNING,
          'MSG_ERROR'   : logging.ERROR,
          'MSG_FATAL'   : logging.CRITICAL}

try: 
    msglevel = os.environ["MSG_REPORT_LEVEL"] 
except:
    msglevel = 'MSG_INFO'

logging.basicConfig(level = LEVELS.get(msglevel, logging.NOTSET),
                    format='%(asctime)s-%(levelname)-s-%(name)s-%(message)s',
                    datefmt='%Y%j%H%M%S')

logger = logging.getLogger('osfile_stretch')

class NoSuchLogical(Exception):
  pass

class NoSuchFile(Exception):
  pass

#--------------------------------------------------------------------------
#
# name:    _expandLogicals
#
# purpose: Converts a filespec, possibly containing a stretched logical,
#          into a list of filespecs, for which files may or may not
#          exist on disk.  Wildcards can be present in the filespec
#          but will not be expanded here.  The logical value can
#          contain another stretched logical.
#
# description:
#          A stretched logical is present if the filespec contains a ":",
#          with the name of the logical being taken from the start of the
#          filespec up to the colon (stretched logicals must ALWAYS be
#          at the beginning of a filespec).
#
#          This routine is not normally called by a user, but is called
#          by other functions within this module that are called by the user.
#
# input:   searchFile - filespec possibly containing a stretched logical
#
# return:  List of filespecs (files may or may not exist, wildcards may
#            be present in the filespecs, all stretched logicals will have been
#            expanded)
#
# exceptions: NoSuchLogical - a filespec is using a stretched logical name
#                             that is undefined
#
# 10/11/06  55866  MSwam  first version
#---------------------------------------------------------------------------
def _expandLogicals (searchFile):
  #
  # initialize with the search spec in the results list
  theList = []
  theList.append(searchFile)
  #
  # repeat until there are no more logicals present
  clean = 0
  while not clean:
    clean = 1  # assume a clean run unless a new logical is found
    msg = "_expandLogicals: theList="+str(theList)
    logger.debug( msg )
    #
    # top of list
    listPos = 0
    listLen = len(theList)
    while listPos < listLen:
      aSearch = theList[listPos]
      #
      # use a regex to parse the string around a separator
      logger.debug( "searching "+aSearch )
      searchResults = STR_SEP_STR_REGEX.search(aSearch)
      if not searchResults:
        # not present, just return search spec unchanged
        logger.debug( "no logical in "+aSearch )
        listPos += 1
        continue

      clean = 0  # logical found so this pass was not clean
      logical_name = searchResults.group(1)  
      filespec = searchResults.group(3)

      try:
        logical_value = os.environ[logical_name]
      except KeyError:
        logger.warn( "No such logical name defined: "+logical_name )
        raise NoSuchLogical

      # parse on whitespace
      logical_parts = logical_value.split()

      # replace the list entry with the first logical value and 
      # insert list entries for each remaining value
      # (each value of a stretched logical is assumed to be a directory
      #  spec so a separator is added for each)
      #
      #
      theList[listPos] = aSearch.replace(
                           logical_name+STRETCHED_LOGICAL_SEPARATOR, 
                           logical_parts[0]+os.sep)
      logger.debug( "aSearch replaced with "+theList[listPos] )
      insertPos = listPos + 1
      for aPart in logical_parts[1:]:
        # NOTE: this is adding entries to the list we are looping through in
        #       the outer "while listPos" block
        #
        logger.debug( "adding "+aPart+os.sep+filespec )
        theList.insert(insertPos, aPart + os.sep + filespec)
        listLen += 1
        insertPos += 1
    # end while in list
  # end while clean
  return theList


#--------------------------------------------------------------------------
#
# name:    findOne
#
# purpose: finds the first existing file in a filespec that could
#          contain a stretched logical, but NO WILDCARDS.
#
# input:   searchFile - filespec possibly containing a stretched logical
#
# return:  First file found using the filespec
#
# exceptions: NoSuchFile - unable to find a matching file
#
# 10/11/06  55866  MSwam  first version
#---------------------------------------------------------------------------
def findOne (searchFile):
  choices = _expandLogicals(searchFile)
  while choices:
    try:
      logger.debug( "findOne: trying to find: "+choices[0] )
      os.stat(choices[0])
    except OSError:
      logger.debug( "found NONE" )
      choices.pop(0)
      continue
    break
  #
  if not choices:
    raise NoSuchFile
  #
  return choices[0]


#--------------------------------------------------------------------------
#
# name:    findAll
#
# purpose: finds all existing files in a filespec that could
#          contain a stretched logical and WILDCARDS.
#
# input:   searchFile - filespec possibly containing a stretched logical
#                       and wildcards
#
# return:  list of files found matching the filespec
#
# exceptions: NoSuchFile - no matching files found
#
# 10/11/06  55866  MSwam  first version
#---------------------------------------------------------------------------
def findAll (searchFile):
  results = []
  specs = _expandLogicals(searchFile)
  for aSpec in specs:
    try:
      logger.debug( "findAll: trying to find: "+aSpec )
      foundFiles = glob.glob(aSpec)
      for aFile in foundFiles:
        logger.debug( "found "+aFile )
        results.append(aFile)
    except:
      logger.debug( "found NONE" )
      continue
  #
  if len(results) == 0:
    raise NoSuchFile
  #
  return results


#========================================================================
# TEST
# % python osfile_stretch.py
#========================================================================
if __name__ == "__main__":
  #
  # find a single file through a stretched logical
  spec = "OPUS_DEFINITIONS_DIR:g2f_pipeline.stage"
  msg = "calling _expandLogicals on "+spec 
  logger.info( msg )
  specs = findOne(spec)
  msg = "expanded = "+str(specs) 
  logger.info( msg )
  #
  # shouldn't expand to anything
  spec = "/should/not/expand"
  msg = "calling _expandLogicals on "+spec 
  logger.info( msg )
  specs = _expandLogicals(spec)
  msg = "expanded = "+str(specs)  
  logger.info( msg )
  #
  # missing logical case
  logger.info( "Should see a WARNING next:" )
  spec = "NO_SUCH_LOGICAL:anything.txt"
  try:
    specs = _expandLogicals(spec)
  except NoSuchLogical:
    logger.info( "AS EXPECTED, no logical found for NO_SUCH_LOGICAL." )
  #
  # a single expansion (no stretch involved)
  spec = "HOME:.envrc"
  logger.info( "calling _expandLogicals on "+spec )
  specs = _expandLogicals(spec)
  logger.info( "expanded = "+str(specs) )
  #
  # wildcard expansion and a stretch, find all of the matches
  spec = "OPUS_DEFINITIONS_DIR:*.stage"
  logger.info( "calling _expandLogicals on "+spec )
  specs = findAll(spec)
  logger.info( "expanded = "+str(specs) )
  #
  logger.info( "Done." )
